% Gradient Reconstruction

close all,clear all, clc
%rand('state',2)

N=32;   % total number of samples
M=8;    % number of missing samples
t = 0:1/N:1-1/N;
n = 0:N-1;

% signal x(n) with sparsity 6
x = 3*sin(2*pi*10*t)+2*cos(2*pi*30*t)+0.5*sin(2*pi*23*t);
figure(1),subplot(321),
h=stem(n,x,'ko');axis([0 N+1 -6 6]);grid on
title('Original signal \itx(n)')

% Random missing sample positions
p = randperm(N);
y = x;
y(p(1:M)) = 0;
y0=y;               % initial y

subplot(322),
h=stem(n(p(M+1:end)),y(p(M+1:end)),'ko');hold on,
h2=stem(n(p(1:M)),y(p(1:M)),'xr','filled');hold off
axis([0 N+1 -6 6]),grid on
title('Available samples')

delta=max(abs(y));

IGG=2;
Mit=60;     % number of iterations (repetitions)
for it=1:Mit
    if it==round(Mit/3), delta=delta/10; end
    if it==round(2*Mit/3), delta=delta/10; end
    
    Y1=zeros(N,1);
    Y2=zeros(N,1);
    for Nx=p(1:M)                          % only for missing samples
        y1=y;
        y1(Nx)=y(Nx)+delta;
        Y1(Nx)=sum((abs(fft(y1))))/N;      % L1-norm (normalised)
        y2=y;
        y2(Nx)=y(Nx)-delta;
        Y2(Nx)=sum((abs(fft(y2))))/N;      % L1-norm (normalised)
    end
    
    Gr=(Y1 - Y2)/(2*delta); % calculate gradient
    yold=y;
    y=y-2*delta*Gr';        % update values    
    ycc(:,it)=y-x;          % calculate error
    
    % graphics
    if it==5 | it==15 | it==Mit
        IGG=IGG+1;
        subplot(3,2,IGG),
        h3=stem(n(p(1:M)),x(p(1:M)));
        set(h3,'color',0.85*[1,1,1],'marker','none','linewidth',2);hold on
        h=stem(n(p(M+1:end)),y(p(M+1:end)),'ko');
        h2=stem(n(p(1:M)),y(p(1:M)),'r');hold off
        axis([0 N+1 -6 6]),
        grid on
        title(['Reconstructed signal in  ',int2str(it),'  iteratons'])
    end
    if it==60, xlabel('time \itn'), end
    
    if max(abs(y-yold))<10^(-5)   % stopping criterion
        break
    end
    
end

subplot(3,2,6),
h=plot(1:Mit,20*log10(mean(abs(ycc')')));axis([0 Mit -80 0]),grid on
xlabel('iteration'),title('Reconstruted MSE in [dB]')